/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.util.custom.config;

import org.slf4j.Logger;
import com.openexchange.annotation.Nullable;
import com.openexchange.config.ConfigurationService;

/**
 * {@link LoggingConfigHolder}
 *
 * @author <a href="mailto:pascal.bleser@open-xchange.com">Pascal Bleser</a>
 * @since v1.2.0
 */
public final class LoggingConfigHolder<C> extends ConfigHolder<C> {

    public static final <X> LoggingConfigHolder<X> with(final Logger log,
        final String name,
        final @Nullable ConfigurationService configurationService,
        final ConfigLoader<X> loader) {
        return new LoggingConfigHolder<X>(log, name, configurationService, loader);
    }

    public static final <X> LoggingConfigHolder<X> with(final Logger log,
        final String name,
        final String prefix,
        final @Nullable ConfigurationService configurationService,
        final ConfigLoader<X> loader) {
        return new LoggingConfigHolder<X>(log, name, prefix, configurationService, loader);
    }
    
    private final Logger log;
    private final String name;
    
    public LoggingConfigHolder(final Logger log,
        final String name,
        final String prefix,
        final @Nullable ConfigurationService configurationService,
        final ConfigLoader<C> loader) {
        super(configurationService, prefix, loader);
        this.log = log;
        this.name = name;
    }
    
    public LoggingConfigHolder(final Logger log,
        final String name,
        final @Nullable ConfigurationService configurationService,
        final ConfigLoader<C> loader) {
        super(configurationService, loader);
        this.log = log;
        this.name = name;
    }

    /**
     * Override this method if you want to be signalled when the configuration
     * has been reloaded and has changed.
     * <p>
     * The default implementation logs an info message.
     * 
     * @param newConfig the new configuration that is now active
     * @since 1.2.0
     */
    @Override
    protected void onConfigChanged(final C newConfig) {
        log.info("loaded new {}", name);
    }

    /**
     * Override this method if you want to be signalled when the configuration
     * has been reloaded and has not changed.
     * <p>
     * The default implementation logs a debug message.
     * @since 1.2.0
     */
    @Override
    protected void onConfigUnchanged() {
        log.debug("{} unchanged", name);
    }

    /**
     * Override this method if you want to be signalled when the configuration
     * has been reloaded and the new configuration is not valid.
     * <p>
     * The default implementation logs an error.
     * @since 1.2.0
     */
    @Override
    protected void onConfigInvalid() {
        log.error("failed to load {}, keeping previous values", name);
    }

}
